/*
   There are two good ways to look at the bytes in
   memory that make up the value stored in a variable.

   One way is to use a C union, which lets us give the
   variable two different types simultaneously. So, for
   example, we can give a variable the type of an int
   and, simultaneously, the type of an array of 4 char.
   The array of 4 char lets us look at the 4 bytes inside
   the int.

   The other way to look at the bytes inside a variable is
   to take the address of the variable and cast that address
   to a pointer to a char. When we dereference the char*, we
   get just one byte of the variable. Then, by doing "pointer
   arithmetic" on the char*, we can move through the individual
   bytes of the variable.
*/
#include <stdio.h>


typedef unsigned char* BytePointer;


typedef union {
   int n;
   unsigned char bytes[sizeof(int)];
} UnpackInt;


typedef union {
   long n;
   unsigned char bytes[sizeof(long)];
} UnpackLong;


typedef union {
   double x;
   unsigned char bytes[sizeof(double)];
} UnpackDouble;



int main()
{
   unsigned int j; // loop index

   UnpackInt    ui;
   UnpackLong   ul;
   UnpackDouble ud;

   int    i;
   long   l;
   double d;

   BytePointer pi = (BytePointer)&i;
   BytePointer pl = (BytePointer)&l;
   BytePointer pd = (BytePointer)&d;

   ui.n = 1234567890;
   ul.n = 1234567890;
   ud.x = 12345.6789;

   i = 1234567890;
   l = 1234567890;
   d = 12345.6789;

   // unpack the int's
   printf("Unpack int %d (%0#10x), two ways.\n", i, i);
   for (j = 0; j < sizeof(int); j++)
   {
      // unpack the int in the union
      printf("%02x  ", ui.bytes[j]);
   }
   printf("\n");
   for (j = 0; j < sizeof(int); j++)
   {
      // unpack an int using a "byte pointer"
      printf("%02x  ", *(pi + j));
   }
   printf("\n");


   // unpack the long's
   printf("Unpack long %ld (%0#*lx), two ways.\n", l, (int)(2+2*sizeof(long)), l);
   for (j = 0; j < sizeof(long); j++)
   {
      // unpack the long in the union
      printf("%02x  ", ul.bytes[j]);
   }
   printf("\n");
   for (j = 0; j < sizeof(long); j++)
   {
      // unpack a long using a "byte pointer"
      printf("%02x  ", *(pl + j));
   }
   printf("\n");


   // unpack the double's
   printf("Unpack double %f, two ways.\n", d);
   for (j = 0; j < sizeof(double); j++)
   {
      // unpack the double in the union
      printf("%02x  ", ud.bytes[j]);
   }
   printf("\n");

   for (j = 0; j < sizeof(double); j++)
   {
      // unpack a double using a "byte pointer"
      printf("%02x  ", *(pd + j));
   }
   printf("\n");

   return 0;
}
